#!/usr/bin/perl -w
##
## actmUtil.pl
##
## (c)  Copyright  IBM Corp.  2004, 2005  All Rights Reserved
##
## US Government Users Restricted Rights - Use, duplication or
## disclosure restricted by GSA ADP Schedule Contract with IBM Corp
##
## Module History:
##  -00 03/05/2004 John Thornton   LM28 initial release
##  -01 03/08/2004 John Thornton   LM29 delete support
##  -02 03/10/2004 John Thornton   LM29 "is directory" support
##  -03 11/01/2004 John Thornton   D61C E6831 - File permission processing
##  -04 02/18/2005 John Thornton   D61N Add copy and execute command functions
##  -05 02/25/2005 John Thornton   D61N Make some error messages go away
##  -06 03/29/2005 John Thornton   D61Q Link processing
##  -07 04/19/2005 John Thornton   D61T Unable to load shared objects during
##                                      execute function
##  -08 05/20/2005 John Thornton   D61Y Patch test fixes
##  -09 08/06/2005 John Thornton   D63H G8651 - '$' files pose problems
##  -10 09/04/2005 John Thornton   D63J H0491 - file permissions lost
##
##

#
# Includes
#
use Getopt::Long;

################################################################################
######                          GLOBAL VARIABLES                          ######
################################################################################

# Command Constants
$UTIL_CMD_NONE          = 0;
$UTIL_CMD_EXISTS        = 1;
$UTIL_CMD_MKDIRS        = 2;
$UTIL_CMD_DELETE        = 3;                                                    #-01
$UTIL_CMD_ISDIR         = 4;                                                    #-02
$UTIL_CMD_PERMS         = 5;                                                    #-03
$UTIL_CMD_COPY          = 6;                                                    #-04
$UTIL_CMD_EXECUTE       = 7;                                                    #-04
$UTIL_CMD_LINK          = 8;                                                    #-06

# Trace Constants
$TRC_T = "XMCLSHLT:";                                                           #-03
$TRC_D = "XMCLSHLD:";
$TRC_F = "XMCLSHLF:";

# Error Constants
$ERROR_ALL_NOERROR            = 0;
$ERROR_ALL_UNKNOWN_COMMAND    = 255;
                                                                                #-01
$ERROR_DELETE_NOTFOUND        = 1;                                              #-01
$ERROR_DELETE_ERROR           = 3;                                              #-01

$ERROR_EXISTS_NOTFOUND        = 1;

$ERROR_ISDIR_NOTFOUND         = 1;                                              #-02

$ERROR_MKDIRS_EXISTS          = 1;
$ERROR_MKDIRS_ISFILE          = 2;
$ERROR_MKDIRS_ERROR           = 3;

$ERROR_PERMS_CHMOD            = 1;                                              #-03
$ERROR_PERMS_CHOWN            = 2;                                              #-03
$ERROR_PERMS_NO_FILE          = 3;                                              #-03
$ERROR_PERMS_FILE_NOT_FOUND   = 4;                                              #-03
                                                                                #-03
$ERROR_COPY_NO_SOURCE_FILE           = 1;                                       #-04
$ERROR_COPY_NO_TARGET_FILE           = 2;                                       #-04
$ERROR_COPY_SOURCE_FILE_NOT_FOUND    = 3;                                       #-04
$ERROR_COPY_EXECUTE                  = 4;                                       #-04
                                                                                #-04
$ERROR_EXEC_NO_COMMAND               = 1;                                       #-04
$ERROR_EXEC_EXECUTE                  = 2;                                       #-04
                                                                                #-04
$ERROR_LINK_NO_TARGET                = 1;                                       #-06
$ERROR_LINK_NO_ALIAS                 = 2;                                       #-06
$ERROR_LINK_TARGET_NOTFOUND          = 3;                                       #-06
$ERROR_LINK_ALIAS_EXISTS             = 4;                                       #-06
$ERROR_LINK_FAILS                    = 5;                                       #-06
$ERROR_LINK_BAD_LINK_TYPE            = 6;                                       #-06
                                                                                #-06
# Program Data
$me = $0;
$me =~ s/^(.*)\///;
$me =~ s/.pl$//;
$cmd = $UTIL_CMD_NONE;
$traceCmd = "actzTrace";

################################################################################
######                          SHARED METHODS                            ######
################################################################################

# start -07
#
# If an error occurs that prevents this function from doing its job, then the
# likely result is that the command execution that required this setup will
# fail. That will be sufficient to reflect this error into the traces and logs.
#
sub createEnvironment
{
   my $this = $me . ".createEnvironment()";
   trace("$TRC_T -> $this");
   my $consoleDir;
   # Many possibilities in the non-production world
   if ($ENV{'CONSOLE_PATH'})
   {
      $consoleDir = $ENV{'CONSOLE_PATH'}.'/';
      trace("$TRC_F    $this - used CONSOLE_PATH");
   }
   elsif ($ENV{'HWMCAHOME'})
   {
      $consoleDir = $ENV{'HWMCAHOME'}.'/';
      trace("$TRC_F    $this - used HWMCAHOME");
   }
   else
   {
      $consoleDir = "/console/";
      trace("$TRC_F    $this - used default");
   }
   # Change any "//" to "/"
   $consoleDir =~ s/\/+/\//g;
   trace("$TRC_F    $this - console root is $consoleDir");
   if ( -x "$consoleDir/hmcfunctions" )
   {
     my $myhmcfunctions = $consoleDir . "hmcfunctions";
     `. $myhmcfunctions; setupEnv $consoleDir`;
     # Nothing, no error is ever returned
   }
   else
   {
     # This should never happen.
     trace("$TRC_F    $this - hmcfunctions not found!");
   }
   trace("$TRC_T <- $this");
}
# end -07

#
#
#
sub trace
{
    my $text = shift;
    chomp($text);

    if ("actzTrace" eq "$traceCmd")
    {
       system(("$traceCmd", "$text"));
    }
    else
    {
       print(STDOUT "$text\n");
    }
}

#
# usage
#
sub usage
{
    print(<<EOM);
Usage: patchUtil [-help] command=cmd [source=sfile] [target=tfile] [...]
   cmd   an integer value that represent a command
            1 - exists()
            2 - mkdirs()
   any other arguments are optional and will vary from command to command
EOM
}

################################################################################
######                          UTILITY METHODS                           ######
################################################################################

# start -01
#
#
sub doDelete
{
   trace("$TRC_D -> $me.doDelete()");
   my $file = shift;

   trace("$TRC_T -> $me.doDelete() - [$file]");                                 #-08

   my $rc = $ERROR_DELETE_NOTFOUND;
   if ( (-e "$file") || (-l "$file") )                                          #-08
   {
      my $error = `rm -f '$file' 2>&1`;                                         #-08-09
      unless (defined($error)) { $error = ""; }                                 #-05
      chop $error;
      $rc = $? >> 8;
      if ($rc)
      {
         trace("$TRC_F    $me.doDelete() - rm error is \"$error\"");
         $rc = $ERROR_DELETE_ERROR
      }
   }
   trace("$TRC_D <- $me.doDelete() - rc = $rc");
   return($rc);
}
# end -01

#
#
#
sub doExists
{
   trace("$TRC_D -> $me.doExists()");
   my $file = shift;

   trace("$TRC_T -> $me.doExists() - [$file]");                                 #-08

   my $rc = $ERROR_EXISTS_NOTFOUND;
   # Does the path directory exist?                                             #-02
   if (-e "$file")
   {
      $rc = $ERROR_ALL_NOERROR;
   }
   trace("$TRC_D <- $me.doExists() - rc = $rc");
   return($rc);
}

# start -02
#
#
sub doIsDir
{
   trace("$TRC_D -> $me.doIsDir()");
   my $file = shift;

   trace("$TRC_T -> $me.doIsDir() - [$file]");                                  #-08

   my $rc = $ERROR_ISDIR_NOTFOUND;
   # Does a directory exist?
   if (-d "$file")
   {
      $rc = $ERROR_ALL_NOERROR;
   }
   trace("$TRC_D <- $me.doIsDir() - rc = $rc");
   return($rc);
}
# end -02

#
#
#
sub doMkDirs
{
   trace("$TRC_D -> $me.doMkDirs()");
   my $file = shift;

   trace("$TRC_T -> $me.doMkDirs() - [$file]");                                 #-08

   my $rc = $ERROR_ALL_NOERROR;
   # Does it already exist?
   my $exists = (-e "$file");
   # Is it a directory?
   my $isDir = (-d "$file");
   if ($exists)
   {
      if ($isDir)
      {
         $rc = $ERROR_MKDIRS_EXISTS;
      }
      else
      {
         $rc = $ERROR_MKDIRS_ISFILE;
      }
   }
   else
   {
      my $error = `mkdir -p '$file' 2>&1`;                                      #-08-09
      unless (defined($error)) { $error = ""; }                                 #-05
      chop $error;
      $rc = $? >> 8;
      if ($rc)
      {
         trace("$TRC_F    $me.doMkDirs() - mkdir error is \"$error\"");
         $rc = $ERROR_MKDIRS_ERROR
      }
   }
   trace("$TRC_D <- $me.doMkDirs() - rc = $rc");
   return($rc);
}

# start -03
#
#
#
sub doPerms
{
   my ($target, $owner, $group, $flags) = @_;                                   #-08

   unless (defined($target)) { $target = ""; }                                  #-08
   unless (defined($owner))  { $owner  = ""; }                                  #-08
   unless (defined($group))  { $group  = ""; }                                  #-08
   unless (defined($flags))  { $flags  = ""; }                                  #-08

   trace("$TRC_T -> $me.doPerms() - [$target], [$owner], [$group], [$flags]");  #-08

   my $rc = $ERROR_ALL_NOERROR;

   # File must be specified
   if ($target eq "")                                                           #-08
   {
      trace("$TRC_F    $me.doPerms() - file argument missing");
      $rc = $ERROR_PERMS_NO_FILE;
   }

   # See if the file/path exists (if no previous error).
   if ($ERROR_ALL_NOERROR == $rc)
   {
      unless ( -e $target )                                                     #-08
      {
         trace("$TRC_F    $me.doPerms() - file [$target] does not exist");      #-08
         $rc = $ERROR_PERMS_FILE_NOT_FOUND;
      }
   }

   # Do the flags portion, if necessary, and if no previous error.
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ($flags ne "")
      {
         my $error = `chmod '$flags' '$target' 2>&1`;                           #-09
         unless (defined($error)) { $error = ""; }                              #-05
         chop $error;
         $rc = $? >> 8;
         if ($rc)
         {
            trace("$TRC_F    $me.doPerms() - chmod error is \"$error\"");
            $rc = $ERROR_PERMS_CHMOD;
         }
      }
   }

   # Do the ownership portion, if necessary, and if no previous error.
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ( ($owner ne "") || ($group ne "") )
      {
         my $error = `chown '$owner:$group' '$target' 2>&1`;                    #-09
         unless (defined($error)) { $error = ""; }                              #-05
         chop $error;
         $rc = $? >> 8;
         if ($rc)
         {
            trace("$TRC_F    $me.doPerms() - chown error is \"$error\"");
            $rc = $ERROR_PERMS_CHOWN;
         }
      }
   }

   trace("$TRC_T <- $me.doPerms() - rc = $rc");
   return($rc);
}
# end -03

# start -04
#
#
#
sub doCopy
{
   my ($source,$target) = @_;

   unless (defined($source)) { $source = ""; }                                  #-05
   unless (defined($target)) { $target = ""; }                                  #-05

   trace("$TRC_T -> $me.doCopy() - [$source], [$target]");

   my $rc = $ERROR_ALL_NOERROR;

   # Source file must be specified
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ($source eq "")
      {
         trace("$TRC_F    $me.doCopy() - source file argument missing");
         $rc = $ERROR_COPY_NO_SOURCE_FILE;
      }
   }

   # Target file must be specified
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ($target eq "")
      {
         trace("$TRC_F    $me.doCopy() - target file argument missing");
         $rc = $ERROR_COPY_NO_TARGET_FILE;
      }
   }

   # See if the source file/path exists
   if ($ERROR_ALL_NOERROR == $rc)
   {
      unless ( -e $source )
      {
         trace("$TRC_F    $me.doCopy() - source file does not exist");
         $rc = $ERROR_COPY_SOURCE_FILE_NOT_FOUND;
      }
   }

   # Attempt the copy.
   if ($ERROR_ALL_NOERROR == $rc)
   {
      my $error = `cp -v --preserve=timestamps '$source' '$target' 2>&1`;       #-09-10
      unless (defined($error)) { $error = ""; }                                 #-05
      chop $error;
      $rc = $? >> 8;
      if ($rc)
      {
         trace("$TRC_F    $me.doCopy() - cp error is \"$error\"");
         $rc = $ERROR_COPY_EXECUTE;
      }
   }

   trace("$TRC_T <- $me.doCopy() - rc = $rc");
   return($rc);
}

#
#
#
sub doExecute
{
   my ($commandString) = @_;

   unless (defined($commandString)) { $commandString = ""; }                    #-05

   trace("$TRC_T -> $me.doExecute() - [$commandString]");

   my $rc = $ERROR_ALL_NOERROR;

   # Command string must be specified
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ($commandString eq "")
      {
         trace("$TRC_F    $me.doExecute() - command string argument missing");
         $rc = $ERROR_EXEC_NO_COMMAND;
      }
   }

   # Attempt the command.
   if ($ERROR_ALL_NOERROR == $rc)
   {
      my $error = `$commandString 2>&1`;
      unless (defined($error)) { $error = ""; }                                 #-05
      chop $error;
      $rc = $? >> 8;
      if ($rc)
      {
         trace("$TRC_F    $me.doExecute() - command rc = $rc - execution error is \"$error\"");  #-07
         $rc = $ERROR_EXEC_EXECUTE;
      }
   }

   trace("$TRC_T <- $me.doExecute() - rc = $rc");
   return($rc);
}
# end -04

# start -06
#
#
#
sub doLink
{
   my ($alias,$target,$symbolic) = @_;

   unless (defined($alias))    { $alias    = ""; }
   unless (defined($target))   { $target   = ""; }
   unless (defined($symbolic)) { $symbolic = ""; }

   trace("$TRC_T -> $me.doLink() - [$alias], [$target]");

   my $rc = $ERROR_ALL_NOERROR;

   # Alias file must be specified
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ($alias eq "")
      {
         trace("$TRC_F    $me.doLink() - alias name argument missing");
         $rc = $ERROR_LINK_NO_ALIAS;
      }
   }

   # Target file must be specified
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ($target eq "")
      {
         trace("$TRC_F    $me.doLink() - target file argument missing");
         $rc = $ERROR_LINK_NO_TARGET;
      }
   }

   # Create the proper value for the symbolic link option
   my $linkType = "";
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ($symbolic eq "")
      {
         $linkType = "";
      }
      elsif ($symbolic eq "true")
      {
         $linkType = "-s";
      }
      else
      {
         trace("$TRC_F    $me.doLink() - unrecognized link type argument: $symbolic");
         $rc = $ERROR_LINK_BAD_LINK_TYPE;
      }
   }

   # See if the target file/path exists
   if ($ERROR_ALL_NOERROR == $rc)
   {
      unless ( -e $target )
      {
         trace("$TRC_F    $me.doLink() - link target file does not exist");
         $rc = $ERROR_LINK_TARGET_NOTFOUND;
      }
   }

   # See if there is already a file where the alias is being created
   if ($ERROR_ALL_NOERROR == $rc)
   {
      if ( -e $alias )
      {
         trace("$TRC_F    $me.doLink() - there is a file where the link was to be created");
         $rc = $ERROR_LINK_ALIAS_EXISTS;
      }
   }

   # Attempt the link.
   if ($ERROR_ALL_NOERROR == $rc)
   {
      my $error = `ln $linkType '$target' '$alias' 2>&1`;                       #-09
      unless (defined($error)) { $error = ""; }
      chop $error;
      $rc = $? >> 8;
      if ($rc)
      {
         trace("$TRC_F    $me.doLink() - cp error is \"$error\"");
         $rc = $ERROR_LINK_FAILS;
      }
   }

   trace("$TRC_T <- $me.doLink() - rc = $rc");
   return($rc);
}
# end -06

################################################################################
######                           SCRIPT CODE                              ######
################################################################################

# Process the arguments
GetOptions(
      'help'            =>\$help,
      'command=i'       =>\$cmd,
      'tracecommand:s'  =>\$traceCmd,
      'target:s'        =>\$target,
# start -04
      'source:s'        =>\$source,
      'commandString:s' =>\$commandString,
# end -04
# start -06
      'alias:s'         =>\$alias,
      'symbolic:s'      =>\$symbolic,
# end -06
# start -03
      'owner:s'         =>\$owner,
      'group:s'         =>\$group,
      'flags:s'         =>\$flags
# end -03
);

trace("$TRC_T -> $me(@ARGV)");

# start -08
# Set up the execution environment, because linux took away our PATH and
# LD_LIBRARY_PATH environment variables for security reasons.
createEnvironment();
# end -08

unless($> == 0) {
   trace("$TRC_F    This must be run as root.");
}

if ($help)
{
   usage();
   exit;
}

my $rc = $ERROR_ALL_NOERROR;

# Route the command
if ($cmd == $UTIL_CMD_EXISTS)
{
   $rc = doExists($target);
}
elsif ($cmd == $UTIL_CMD_MKDIRS)
{
   $rc = doMkDirs($target);
}
elsif ($cmd == $UTIL_CMD_DELETE)                                                #01
{                                                                               #01
   $rc = doDelete($target);                                                     #01
}                                                                               #01
elsif ($cmd == $UTIL_CMD_ISDIR)                                                 #02
{                                                                               #02
   $rc = doIsDir($target);                                                      #02
}                                                                               #02
elsif ($cmd == $UTIL_CMD_PERMS)                                                 #03
{                                                                               #03
   $rc = doPerms($target,$owner,$group,$flags);                                 #03
}                                                                               #03
elsif ($cmd == $UTIL_CMD_COPY)                                                  #04
{                                                                               #04
   $rc = doCopy($source,$target);                                               #04
}                                                                               #04
elsif ($cmd == $UTIL_CMD_EXECUTE)                                               #04
{                                                                               #04
   $rc = doExecute($commandString);                                             #04
}                                                                               #04
elsif ($cmd == $UTIL_CMD_LINK)                                                  #06
{                                                                               #06
   $rc = doLink($alias,$target,$symbolic);                                      #06
}                                                                               #06
else
{
   # Unknown command
   $rc = $ERROR_ALL_UNKNOWN_COMMAND;
}

trace("$TRC_T <- $me() - rc = $rc");

exit($rc);
